#include <stddef.h>
#include <string.h>

#include <avr/interrupt.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/power.h>
#include <avr/wdt.h>
#include <util/delay.h>

#include "bios.h"
#include "boot.h"
#include "canboot.h"
#include "candobus/bus.h"
#include "candobus/frame.h"
#include "format/hex2bin.h"
#include "hal.h"
// #include "debug.h"

#define bios_kBootTimeout (0x4000)

uint8_t bios_buffer[bus_kBufferSize] __attribute__((section(".noinit")));

uint8_t *bios_getNodeName(void) {
  boot_stradd_P(bios_buffer, nodeConfig.nodeName);
  return bios_buffer;
}

void jumpTable(void) __attribute__((naked))
__attribute__((section(".jumptable")));
void jumpTable(void) { canboot_jumpTable; }

int main(void) __attribute__((OS_main)) __attribute__((section(".init9")))
__attribute__((noreturn));

int main(void) {
  uint16_t csum = 0;
  uint16_t bootTimeout = bios_kBootTimeout / 2;

  // initialise micro as this code has been excluded using -nostartfiles
  asm volatile("clr __zero_reg__"); // clear zero register
  SREG = 0;    // clear status register - ensure interrupts are disabled
  SP = RAMEND; // initialise stack pointer

  watchDogReset(); /* reset and enable watchdog */
  WDTCSR = (1 << WDCE) | (1 << WDE);
  WDTCSR = WDTO_250MS | (1 << WDE);

  bus_init();

  while (1) {

    boot_sendResponse(bios_buffer, NULL);

    // main bootloader loop
    while (bootTimeout--) {
      _delay_us(50);

      // give 3 red flashes at 40 ms on 120 ms off
      if ((bootTimeout < 0x2000) && csum) {
        if (((bootTimeout & 0xFE00) == 0x0000) ||
            ((bootTimeout & 0xFE00) == 0x0C00) ||
            ((bootTimeout & 0xFE00) == 0x1800)) {
          hal_uiRedOn();
        } else {
          hal_uiRedOff();
        }
      }

      uint8_t status = boot_read(bios_buffer);
      if (canboot_kBootActive == status) {
        bootTimeout = bios_kBootTimeout;
        if ((csum++ & 0x03) == 0) {
          hal_uiStep();
        }
      }
    }

    // release use of LED for bootloader UI
    hal_uiDisable();

    // calculate application check sum
    csum = 0;
    PGM_P app = pgm_read_word(TEXTSTART - 4);
    while (app--) {
      csum += pgm_read_byte(app);
    }

    // if application checksum is good start app
    if (csum == pgm_read_word(TEXTSTART - 2)) {
      watchDogReset();
      startApp();
    }

    bootTimeout = bios_kBootTimeout;
  }
}
