#include <string.h>

#include "bios.h"
#include "boot.h"
#include "prog.h"

#include "candobus/bus.h"
#include "candobus/parse.h"

const node_t __attribute__((section(".nodeconfig"))) nodeConfig = node_config;

// passed pointer to parameter string
// ensures the character before the returned pointer is a forward slash
// returned pointer to next parameter or terminating NULL character if no more
// parameters
uint8_t *boot_nextParameter(uint8_t *data) { return parse_next(data); }

uint8_t boot_write(uint8_t *buffer, uint8_t priority) {
  return bus_write(buffer, priority);
}

// like strcat_P but returns a pointer to the terminating null
// making it easier to build up strings without recourse to strcat() or strlen()
uint8_t *boot_stradd_P(uint8_t *str, PGM_P pstr) {
  uint8_t ch;
  do {
    ch = pgm_read_byte(pstr++);
    *str++ = ch;
  } while (0 != ch);
  return --str;
}

uint8_t boot_strmatch_P(uint8_t *str, PGM_P pstr) {
  uint8_t buffer[9];
  boot_stradd_P(buffer, pstr);
  return parse_match(str, buffer);
}

void boot_sendResponse(uint8_t *buffer, PGM_P response) {
  uint8_t priority = bus_kLowPriority;
  uint8_t *str = boot_stradd_P(buffer, nodeConfig.nodeName);
  if (response) {
    str = boot_stradd_P(str, nodeConfig.boot_P);
    str = boot_stradd_P(str, response);
    priority = bus_kResponsePriority;
  }
  bus_write(buffer, priority);
}

uint8_t boot_read(uint8_t *buffer) {
  watchDogReset();
  uint8_t status = bus_read(buffer);
  if (bus_kRXCompleted == status) {
    uint8_t match = boot_strmatch_P(buffer, nodeConfig.nodeName);
    if (parse_kDifferent != match) {
      uint8_t *params = boot_nextParameter(buffer);
      if ('!' == *params) {
        // pass to bootloader
        boot_sendResponse(buffer, prog_parse(params));
        status = canboot_kBootActive;
      } else if (parse_kSimilar == match) {
        boot_sendResponse(buffer, NULL);
        // send node name to search response
        status = canboot_kBootActive;
      } else {
        status = canboot_kNodeMatch;
      }
    }
  }
  return status;
}
