#include 	<avr/io.h>

#include  "flash.h"

#define	PAGE_MASK	(SPM_PAGESIZE - 1)
#define	BLOCK_MASK	(flash_kBlockSize - 1)
#define	WRITE_MASK	(PAGE_MASK^BLOCK_MASK)
#define	BLOCK_WORDS	(flash_kBlockSize/2)

#define	STATUS		r20
#define	TEMP		r21

#ifndef SELFPRGEN
#define SELFPRGEN SPMEN
#endif

#ifndef SPMCSR
#define SPMCSR SPMCR
#endif

.section .text
.global flash_write
.global flash_enable


flash_write:
	cpi		r25, hi8(TEXTSTART)				// disable writing to boot flash
	brsh	exit
	movw	XL, r22							// copy data pointer to X
	movw	ZL, r24							// copy write address to Z
	ldi		r18, BLOCK_WORDS				// initialize word counter

copyToBuffer:
	ld		r0, X+							// copy word to flash write buffer
	ld		r1, X+
	ldi		TEMP, (1<<SPMEN)
	rcall	writeSPM
	adiw	ZL, 2							// move to next word
	dec		r18
	brne	copyToBuffer

	movw	ZL, r24							// copy write address to Z

	andi	r24, PAGE_MASK
	breq	erasePage						// if first block of page then erase
	cpi		r24, WRITE_MASK
	brne	exit							// if last block of page then write page

writePage:
	ldi		TEMP, (1<<PGWRT) | (1<<SPMEN)	// execute page write
	rjmp	writeSPM

erasePage:
	ldi		TEMP, (1<<PGERS) | (1<<SPMEN)	// execute page erase
	rjmp	writeSPM

flash_enable:
	ldi		TEMP, (1<<RWWSRE) | (1<<SPMEN)	// re-enable the RWW section

writeSPM:
	lds		STATUS, SPMCSR					// check for previous SPM complete
	sbrc	STATUS, SELFPRGEN
	rjmp	writeSPM
	cli
	sts		SPMCSR, TEMP					// execute spm with action given by TEMP
	spm
	sei
exit:
	clr		r1
	ret
