#ifndef HAL_JACKAL_H_
#define HAL_JACKAL_H_

#if defined(__cplusplus)
extern "C" {
#endif

#include <avr/interrupt.h>
#include <avr/io.h>
#include <avr/wdt.h>
#include <stdint.h>

#define BAUD 9600L
#include <util/setbaud.h>

#define NODE_STR "div"

#define bus_kBufferSize (64)

#define hal_k20BitPeriods ((20 * F_CPU) / BAUD / 256L)

extern uint8_t hal_busChangeTime __attribute__((section(".noinit")));
extern uint8_t hal_busInactiveTimer __attribute__((section(".noinit")));

extern uint8_t bus_status __attribute__((section(".noinit")));
extern uint8_t bus_buffer[bus_kBufferSize] __attribute__((section(".noinit")));
extern uint8_t *bus_bufferPtr __attribute__((section(".noinit")));
extern uint8_t bus_holdoff __attribute__((section(".noinit")));

#define hal_getInactiveTimer()                                                 \
  ({                                                                           \
    uint8_t t = TCNT0;                                                         \
    if (EIFR & (1 << INTF0)) {                                                 \
      EIFR |= (1 << INTF0);                                                    \
      hal_busChangeTime = t;                                                   \
      hal_busInactiveTimer = 0;                                                \
    } else {                                                                   \
      if (hal_busInactiveTimer < 128) {                                        \
        hal_busInactiveTimer = t - hal_busChangeTime;                          \
      }                                                                        \
    }                                                                          \
    hal_busInactiveTimer;                                                      \
  })

// should return value of timer required for multiples of approx 10 bits
#define hal_random(p) (hal_k20BitPeriods + (p << 4) + (TCNT0 & 0x0F))

#define hal_status() (UCSR0A)

#define hal_rxComplete(s) (s & (1 << RXC0))
#define hal_rxError(s) (s & ((1 << DOR0) | (1 << FE0) | (1 << UPE0)))
#define hal_rx() (UDR0)

#define hal_txComplete(s) (s & (1 << TXC0))
#define hal_txClear(s) (s &= ~(1 << TXC0))

#define hal_tx(x)                                                              \
  do {                                                                         \
    UDR0 = x;                                                                  \
    UCSR0A |= (1 << TXC0);                                                     \
  } while (0)

#define hal_init()                                                             \
  do {                                                                         \
    TCCR0B = (1 << CS02); /* prescale by 256, 14400 Hz timer*/                 \
    UBRR0L = UBRRL_VALUE; /* uart at 9600 8n1 */                               \
    UCSR0B = (1 << RXEN0) | (1 << TXEN0);                                      \
    EICRA = (1 << ISC00); /* activate interrupt flag on change of rx pin */    \
  } while (0)

#define hal_uiOn()                                                             \
  do {                                                                         \
    DDRD |= (1 << PORTD5) | (1 << PORTD7);                                     \
    PORTD |= (1 << PORTD7);                                                    \
    PORTD &= ~(1 << PORTD5);                                                   \
  } while (0)

#define hal_uiOff()                                                            \
  do {                                                                         \
    DDRD |= (1 << PORTD5) | (1 << PORTD7);                                     \
    PORTD &= ~(1 << PORTD7);                                                   \
    PORTD |= (1 << PORTD5);                                                    \
  } while (0)

#define hal_uiToggle()                                                         \
  do {                                                                         \
    DDRD |= (1 << PORTD5) | (1 << PORTD7);                                     \
    PORTD ^= (1 << PORTD5) | (1 << PORTD7);                                    \
  } while (0)

#define hal_uiDisable()                                                        \
  do {                                                                         \
    DDRD &= ~((1 << PORTD5) | (1 << PORTD7));                                  \
    PORTD &= ~((1 << PORTD5) | (1 << PORTD7));                                 \
  } while (0)

#if defined(__cplusplus)
}
#endif

#endif /* HAL_JACKAL_H_ */
