#include "candobus/frame.h"
#include "format/crypt.h"
#include "format/hex2bin.h"

#include "bios.h"
#include "flash.h"
#include "prog.h"

PGM_P prog_parse(uint8_t *str) {
  uint8_t *params = boot_nextParameter(str);
  if (*params) {
    if (22 == hex2bin(params)) {
      uint16_t salt = *((uint16_t *)params);
      if (salt)
        crypt_block((uint8_t *)(params + 2), salt);
      if ('F' == *(params + 2)) {
        // program flash
        if (18 == *(params + 3)) {
          uint16_t address = *((uint16_t *)(params + 4));
          flash_write(address, params + 6);
          return nodeConfig.ok_P;
        }
      }
    } else if ('B' == *params) {
      // return build timestamp (0x42)
      return nodeConfig.nodeBuild;
    } else if ('S' == *params) {
      // return serial number (0x53)
      return nodeConfig.nodeSerial;
    } else if ('R' == *params) {
      // reboot (0x52)
      _delay_us(5000);
      flash_enable();
      while (1) {
        ;
      }
    }
  }
  return nodeConfig.err_P;
}
