#ifndef Arduino_h
#define Arduino_h

#include <math.h>
#include <stdbool.h>
#include <stdlib.h>
#include <string.h>

#include <avr/interrupt.h>
#include <avr/io.h>
#include <avr/pgmspace.h>

#ifdef __cplusplus
extern "C" {
#endif

#include "canboot.h"
#include "variant.h"

#include "protothreads/pt.h"

#define wiring_kMaxPWMCount (F_CPU / 800UL)

#define HIGH 0x1
#define LOW 0x0

#define INPUT 0x0
#define OUTPUT 0x1
#define INPUT_PULLUP 0x2

void init(void);
void initVariant(void);
void loopVariant(void);

unsigned long micros(void);
void delay(unsigned long ms);

void pinMode(uint8_t pin, uint8_t value);
void digitalWrite(uint8_t pin, uint8_t value);
int digitalRead(uint8_t pin);

int analogRead(uint8_t channel);
void analogWrite(uint8_t channel, int value);

int eepromRead(uint8_t address);
void eepromWrite(uint8_t address, int value);
void eepromUpdate(uint8_t address, int value);

uint8_t *getRequest(void);
bool sendResponse(uint8_t *str);
bool sendRequest(PGM_P str, uint8_t priority);

int8_t strmatch_P(uint8_t *str, PGM_P pstr);
uint8_t *strparam(uint8_t *tr);

enum itoa_format {
  Unit = 0,
  Deci,
  Centi,
  Milli,
  TwoDigits,
  Time,
};

enum itoa_constants { itoa_kMaxStringLength = 8 };

uint8_t intToString(uint8_t *str, int16_t value, uint8_t format);

void setup(void);
void loop(void);

#ifdef __cplusplus
} // extern "C"
#endif

#include "pins_arduino.h"

#endif
