#ifndef CANBOOT_H
#define CANBOOT_H

#if defined(__cplusplus)
extern "C" {
#endif

// size of buffer used by the bus and returned by getNodeName
#define canboot_kBufferSize (64)

// message priority
#define canboot_kLowPriority (3) // lowest priority - maximum wait
#define canboot_kMedPriority (2)
#define canboot_kHighPriority (1)     // highest priority - minimum wait
#define canboot_kResponsePriority (0) // send responses immediately

// return status values from read, write and match functions
#define canboot_kRXReceiving (0)
#define canboot_kRXCompleted (1)
#define canboot_kRXAborted (2)
#define canboot_kTXSending (3)
#define canboot_kTXCompleted (5)
#define canboot_kTXAborted (6)

#define canboot_kBootActive (7)
#define canboot_kNodeMatch (8)

#define canboot_kSame (0)
#define canboot_kSimilar (1)
#define canboot_kDifferent (2)

// jump table entry points
#define JUMPTABLE (JUMPSTART >> 1)

#define GETNODENAME (JUMPTABLE)
#define BUSREAD (JUMPTABLE + 1)
#define BUSWRITE (JUMPTABLE + 2)
#define STRMATCH (JUMPTABLE + 3)
#define STRNEXT (JUMPTABLE + 4)

#define canboot_getNodeName ((uint8_t * (*)(void)) GETNODENAME)
#define canboot_busRead ((uint8_t(*)(uint8_t * str)) BUSREAD)
#define canboot_busWrite                                                       \
  ((uint8_t(*)(uint8_t * str, uint8_t priority)) BUSWRITE)
#define canboot_strMatch ((uint8_t(*)(uint8_t * str, PGM_P pstr)) STRMATCH)
#define canboot_strNext ((uint8_t * (*)(uint8_t * str)) STRNEXT)

#if defined(__cplusplus)
}
#endif

#endif /* CANBOOT_H_ */
