#include <avr/interrupt.h>
#include <avr/io.h>
#include <stdio.h>

#include "Arduino.h"

#ifndef TIMSK1
#define TIMSK1 TIMSK
#endif

#ifndef ICIE1
#define ICIE1 TICIE1
#endif

#define wiring_kadcChannelMask (0b0111)

enum { wiring_kUp = 0, wiring_kDown };

static volatile uint32_t wiring_cycles = 0;
static volatile uint8_t wiring_duty = 0;
static volatile uint8_t wiring_direction = wiring_kUp;
static uint8_t *canboot_buffer;

void __cxa_pure_virtual() {
  while (1) {
  }
}

unsigned long micros(void) {
  cli();
  uint32_t tmr = TCNT1;
  uint32_t cnt = wiring_cycles;
  uint8_t dir = wiring_direction;
  sei();

  if (dir == wiring_kUp) {
    cnt += tmr;
  } else {
    cnt += wiring_kMaxPWMCount - tmr;
  }
  return cnt;
}

void delay(unsigned long ms) {
  uint16_t start = (uint16_t)micros();
  while (ms > 0) {
    if (((uint16_t)micros() - start) >= 1000) {
      ms--;
      start += 1000;
    }
  }
}

void sleep() __attribute__((weak));
void sleep() {}

uint8_t *getRequest(void) {
  // check for new message on the bus
  uint8_t cmp = canboot_busRead(canboot_buffer);

  if ((canboot_kNodeMatch == cmp)) {
    return canboot_buffer;
  } else {
    return NULL;
  }
}

bool sendResponse(uint8_t *str) {
  // int8_t status;
  // do {
  canboot_busWrite(str, canboot_kResponsePriority);
  //  do {
  //    status = canboot_busRead(str);
  //  } while (canboot_kTXSending == status);
  //} while (canboot_kTXCompleted != status);
  return true;
}

bool sendRequest(PGM_P str, uint8_t priority) {
  strcpy_P((char *)canboot_buffer, str);
  return canboot_busWrite(canboot_buffer, priority);
}

void init() {
  canboot_buffer = canboot_getNodeName();
  /*  strcat_P((char *)canboot_buffer, PSTR("up"));*/
  /*  canboot_busWrite(canboot_buffer, canboot_kLowPriority);*/
}

int _itoa(uint8_t *str, uint8_t format, int16_t value) {
  uint8_t digit;
  uint8_t enable_digits = 0;
  uint8_t *ptr;

  ptr = str;

  // do sign
  if (value < 0) {
    value = -value;
    *str++ = '-';
  }

  // do 10000's
  digit = '0';
  while (value > 9999) {
    value -= 10000;
    digit++;
  }
  if ((digit != '0') || (enable_digits != 0)) {
    *str++ = digit;
    enable_digits = 1;
  }

  // do 1000's
  digit = '0';
  while (value > 999) {
    value -= 1000;
    digit++;
  }
  if ((digit != '0') || (enable_digits != 0)) {
    *str++ = digit;
    enable_digits = 1;
  }

  // do 100's
  digit = '0';
  while (value > 99) {
    value -= 100;
    digit++;
  }
  if (format == Milli) {
    if (enable_digits == 0)
      *str++ = '0';
    *str++ = '.';
    enable_digits = 1;
  }
  if ((digit != '0') || (enable_digits != 0)) {
    *str++ = digit;
    enable_digits = 1;
  }

  // do 10's
  digit = '0';
  while (value > 9) {
    value -= 10;
    digit++;
  }
  if (format == Centi) {
    if (enable_digits == 0)
      *str++ = '0';
    *str++ = '.';
    enable_digits = 1;
  }
  if ((digit != '0') || (enable_digits != 0) || (format == TwoDigits)) {
    *str++ = digit;
    enable_digits = 1;
  }

  // do 1's
  digit = '0' + value;
  if (format == Deci) {
    if (enable_digits == 0)
      *str++ = '0';
    *str++ = '.';
  }
  *str++ = digit;

  // terminate string and return
  *str = 0;
  return str - ptr;
}

/// Convert integer to ascii decimal string and send to terminal
/// @param integer value
/// @param desired ascii decimal format
uint8_t intToString(uint8_t *str, int16_t value, uint8_t format) {
  int16_t hours = 0;

  if (format == Time) {
    while (value >= 60) {
      ++hours;
      value -= 60;
    }
    str += _itoa(str, TwoDigits, hours);
    *str++ = '.';
    _itoa(str, TwoDigits, value);
  } else {
    _itoa(str, format, value);
  }
  return strlen((char *)str);
}
