#ifndef LOGGER_H_
#define LOGGER_H_

#include <string.h>

#include "Arduino.h"
#include "INode.h"

#include "utility/asm.h"
#include "utility/hex2bin.h"
#include "utility/log.h"

const char PROGMEM Logger_kReadString[] = "read";
const char PROGMEM Logger_kStatusString[] = "status";
const char PROGMEM Logger_kResetString[] = "reset";
const char PROGMEM Logger_kStreamString[] = "stream";
const char PROGMEM Logger_kResourceString[] = "log";

class Logger : public INode {
public:
  void processRequest(uint8_t *request) {
    if (*request) {
      if (canboot_kSame == canboot_strMatch(request, Logger_kReadString)) {
        // respond with page of binary data from buffer
        // start retrieving next page of data
        request = canboot_strNext(request);
        memcpy(request, &readBuffer, sizeof(readBuffer));
        bin2hex(request, sizeof(readBuffer));
        log_read(&readBuffer);
      } else if (canboot_kSame ==
                 canboot_strMatch(request, Logger_kResetString)) {
        // reset read to last logger record
        log_reset();
        log_read(&readBuffer);
        stream = false;
        request = canboot_strNext(request);
        strcpy_P((char *)request, INode_kOKString);
      } else if (canboot_kSame ==
                 canboot_strMatch(request, Logger_kStreamString)) {
        // send live data to serial
        stream = true;
        request = canboot_strNext(request);
        strcpy_P((char *)request, INode_kOKString);
      } else if (canboot_kSame ==
                 canboot_strMatch(request, Logger_kStatusString)) {
        // respond with status information
        request = canboot_strNext(request);
        *((uint32_t *)request) =
            ((uint32_t)(asm_UTIL_htons(readAddress)) << 16L) |
            ((uint32_t)(asm_UTIL_htons(writeAddress)));
        bin2hex(request, sizeof(uint32_t));
      } else {
        request = canboot_strNext(request);
        strcpy_P((char *)request, INode_kNotFoundString);
      }
    } else {
      strcpy_P((char *)request, INode_kOpenListString);
      strcat_P((char *)request, Logger_kReadString);
      strcat_P((char *)request, INode_kListSeperatorString);
      strcat_P((char *)request, Logger_kResetString);
      strcat_P((char *)request, INode_kListSeperatorString);
      strcat_P((char *)request, Logger_kStreamString);
      strcat_P((char *)request, INode_kListSeperatorString);
      strcat_P((char *)request, Logger_kStatusString);
      strcat_P((char *)request, INode_kCloseListString);
    }
  }

  PGM_P getName() { return Logger_kResourceString; }

  int8_t write(nv_blob_t *blob) {
    // data = asm_UTIL_htonl(data);
    if (stream) {
      // output log write data to bus
      uint8_t *canboot_buffer = canboot_getNodeName();
      strcat_P((char *)canboot_buffer, PSTR("/"));
      strcat_P((char *)canboot_buffer, Logger_kResourceString);
      strcat_P((char *)canboot_buffer, PSTR("/"));
      uint8_t buffer[48] = {0};
      memcpy(buffer, blob, log_kRecordSize);
      bin2hex(buffer, log_kRecordSize);
      strcat((char *)canboot_buffer, (char *)buffer);
      canboot_busWrite(canboot_buffer, canboot_kLowPriority);
    }
    return log_write(blob);
  }

  void begin(void) {
    log_init();
    stream = false;
  }

  int8_t update() { return log_update(); }

private:
  bool stream;
  nv_blob_t readBuffer;
};

Logger logger;

#endif /* LOGGER_H_ */
