#include <string.h>

#include "nv.h"

#define nv_kDeviceAddress 0x50 // 0b0101 0 a2 a1 a0

#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wunused-but-set-variable"

// leave space at top of device for counter rollover
const uint16_t deviceSize = 0x3FFF - nv_kBlobSize;

static struct pt pt;
typedef PT_THREAD((*updateFuncPtr)(void));

typedef struct {
  uint8_t memoryAddr[2];
  nv_blob_t blob;
} twiBuffer_t;

static twiBuffer_t twi;
static uint8_t deviceAddr;
static nv_blob_t *readDest;

uint16_t nv_getSize(void) { return deviceSize; }

void nv_setAdrress(uint16_t address) {
  twi.memoryAddr[0] = (address >> 4) & 0xFF; // address msb
  twi.memoryAddr[1] = (address << 4) & 0xFF; // address lsb
  deviceAddr = nv_kDeviceAddress | (address >> 12);
}

PT_THREAD(nv_read(uint16_t address, nv_blob_t *blob)) {
  PT_BEGIN(&pt);
  // set address and destination for read data
  nv_setAdrress(address);
  readDest = blob;
  // start dummy write to set address
  twi_write(deviceAddr, twi.memoryAddr, sizeof(twi.memoryAddr));
  // wait for write to complete
  PT_WAIT_WHILE(&pt, twi_getStatus() == TWI_BUSY);
  // start read data
  twi_read(deviceAddr, readDest->data, nv_kBlobSize);
  // wait for read to complete
  PT_WAIT_WHILE(&pt, twi_getStatus() == TWI_BUSY);
  PT_END(&pt);
}

PT_THREAD(nv_isFree(uint16_t address)) {
  PT_BEGIN(&pt);
  // set address and destination for read data
  nv_setAdrress(address);
  readDest = &twi.blob;
  // start dummy write to set address
  twi_write(deviceAddr, twi.memoryAddr, sizeof(twi.memoryAddr));
  // wait for write to complete
  PT_WAIT_WHILE(&pt, twi_getStatus() == TWI_BUSY);
  // start read data
  twi_read(deviceAddr, readDest->data, nv_kBlobSize);
  // wait for read to complete
  PT_WAIT_WHILE(&pt, twi_getStatus() == TWI_BUSY);
  // test for blank page
  uint8_t i = nv_kBlobSize;
  while (i--) {
    if (twi.blob.data[i] != 0xFF) {
      PT_EXIT(&pt);
    }
  }
  PT_END(&pt);
}

PT_THREAD(nv_write(uint16_t address, nv_blob_t *blob)) {
  PT_BEGIN(&pt);
  // set address and copy write data
  nv_setAdrress(address);
  memcpy(&twi.blob, blob, nv_kBlobSize);
  // start write data
  twi_write(deviceAddr, (uint8_t*)&twi, sizeof(twi));
  // wait for write to complete
  PT_WAIT_WHILE(&pt, twi_getStatus() == TWI_BUSY);
  PT_END(&pt);
}

PT_THREAD(nv_erase(uint16_t address)) {
  PT_BEGIN(&pt);
  // set address and clear write data
  nv_setAdrress(address);
  memset(&twi.blob, 0xFF, nv_kBlobSize);
  // start write data
  twi_write(deviceAddr, (uint8_t*)&twi, sizeof(twi));
  // wait for write to complete
  PT_WAIT_WHILE(&pt, (twi_getStatus() == TWI_BUSY));
  PT_END(&pt);
}

void nv_init(void) {
  twi_init();
  PT_INIT(&pt);
}

#pragma GCC diagnostic pop
