#include "frame.h"

#define CR ('\r')
#define LF ('\n')
#define ESC (0x1B)

// passed string in format "{DATA}"
// returns string in format ":{DATA}{CR}{ESC}[>{CSUM}{CSUM}{LF}"
void frame_enframe(uint8_t *dst, uint8_t *src) {
  uint8_t csum = 0;

  *dst++ = ':';

  while (*src) {
    csum += *src;
    *dst++ = *src++;
  }

  *dst++ = CR;
  *dst++ = ESC;
  *dst++ = '[';
  *dst++ = '>';
  *dst++ = '0' + ((csum >> 4) & 0x07);
  *dst++ = 'A' + (csum & 0x0F);
  *dst++ = LF;
  *dst++ = 0;
}

// passed string in format ":{DATA}{CR}{ESC}[>{CSUM}{CSUM}{LF}" or
// ":{DATA}{CR}{LF}" modifies string to be in format "{DATA}" and returns 0xFF
// if the check sum is OK or not present
uint8_t frame_deframe(uint8_t *dst, uint8_t *src) {
  uint8_t csum = 0;

  if (*src++ != ':') {
    return 0x00;
  }
  do {
    if (*src == CR) {
      src++;
      *dst = 0x00;
      if (*src == LF) {
        return 0xFF;
      }
      if (*src++ != ESC) {
        return 0x00;
      }
      if (*src++ != '[') {
        return 0x00;
      }
      if (*src++ != '>') {
        return 0x00;
      }
      uint8_t chr = *src++ - ((csum >> 4) & 0x07);
      if (chr != '0') {
        return 0x00;
      }
      chr = *src++ - (csum & 0x0F);
      if (chr != 'A') {
        return 0x00;
      }
      if (*src == LF) {
        return 0xFF;
      }
      return 0x00;
    }
    *dst++ = *src;
    csum += *src++;
  } while (*src);

  return 0x00;
}
