#ifndef VARIANT_H_
#define VARIANT_H_

#ifdef __cplusplus
extern "C" {
#endif
#include "avr/io.h"

#include "led/led.h"

#define kLineFrequency (50UL)

#define kMaxLinePeriod (F_CPU / (45.0 * 8.0))
#define kSampleDelay (F_CPU / (125.0 * 8.0))

#define kMinLinePeriod (F_CPU / (55.0 * 8.0))
#define kNomLinePeriod (F_CPU / (50.0 * 8.0))
#define kBlankPeriod (F_CPU / (500.0 * 8.0))

#define kTestDutyCycle -333
#define kFullDutyCycle -666
#define kZeroDutyCycle 0

#define DEBUG_MISO(x)                                                          \
  do {                                                                         \
    DDRB |= (1 << PORTB4);                                                     \
    uint8_t y = x;                                                             \
    while (y--) {                                                              \
      PORTB |= (1 << PORTB4);                                                  \
      PORTB &= ~(1 << PORTB4);                                                 \
    }                                                                          \
  } while (0)

#define DEBUG_MOSI(x)                                                          \
  do {                                                                         \
    DDRB |= (1 << PORTB3);                                                     \
    uint8_t y = x;                                                             \
    while (y--) {                                                              \
      PORTB |= (1 << PORTB3);                                                  \
      PORTB &= ~(1 << PORTB3);                                                 \
    }                                                                          \
  } while (0)

#define SSR_STATE (PORTB & (1 << PORTB1))

#define SSR_ON()                                                               \
  do {                                                                         \
    DDRB |= (1 << PORTB1);                                                     \
    PORTB |= (1 << PORTB1);                                                    \
  } while (0)

#define SSR_OFF()                                                              \
  do {                                                                         \
    DDRB |= (1 << PORTB1);                                                     \
    PORTB &= ~(1 << PORTB1);                                                   \
  } while (0)

#define GREEN_ON()                                                             \
  do {                                                                         \
    DDRD |= (1 << PORTD6);                                                     \
    PORTD |= (1 << PORTD6);                                                    \
  } while (0)

#define GREEN_OFF()                                                            \
  do {                                                                         \
    DDRD |= (1 << PORTD6);                                                     \
    PORTD &= ~(1 << PORTD6);                                                   \
  } while (0)

#define AMBER_ON()                                                             \
  do {                                                                         \
    DDRD |= (1 << PORTD5);                                                     \
    PORTD |= (1 << PORTD5);                                                    \
  } while (0)

#define AMBER_OFF()                                                            \
  do {                                                                         \
    DDRD |= (1 << PORTD5);                                                     \
    PORTD &= ~(1 << PORTD5);                                                   \
  } while (0)

#define ATN0_ON()                                                              \
  do {                                                                         \
    DDRC |= (1 << PORTC2);                                                     \
    PORTC |= (1 << PORTC2);                                                    \
  } while (0)

#define ATN0_OFF()                                                             \
  do {                                                                         \
    DDRC |= (1 << PORTC2);                                                     \
    PORTC &= ~(1 << PORTC2);                                                   \
  } while (0)

#define ATN1_ON()                                                              \
  do {                                                                         \
    DDRC |= (1 << PORTC3);                                                     \
    PORTC |= (1 << PORTC3);                                                    \
  } while (0)

#define ATN1_OFF()                                                             \
  do {                                                                         \
    DDRC |= (1 << PORTC3);                                                     \
    PORTC &= ~(1 << PORTC3);                                                   \
  } while (0)

#define ATN_48V()                                                              \
  do {                                                                         \
    ATN0_ON();                                                                 \
    ATN1_OFF();                                                                \
  } while (0)

#define ATN_24V()                                                              \
  do {                                                                         \
    ATN0_OFF();                                                                \
    ATN1_ON();                                                                 \
  } while (0)

#define ATN_12V()                                                              \
  do {                                                                         \
    ATN0_OFF();                                                                \
    ATN1_OFF();                                                                \
  } while (0)

#ifdef __cplusplus
} // extern "C"
#endif

#endif /* VARIANT_H_ */
